//+------------------------------------------------------------------+
//| HeikenAshiOscillator.mq4
//| Copyright http://www.pointzero-indicator.com
//+------------------------------------------------------------------+

#property copyright "Copyright  Pointzero-indicator.com"
#property link      "http://www.pointzero-indicator.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  DodgerBlue
#property indicator_color2  Red
#property indicator_color3  Teal
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  1

//-------------------------------
// External variables
//-------------------------------
//
//

extern string TimeFrame           = "Current time frame";
extern int    AvPeriod            = 2;
extern int    MaMethod            = MODE_SMA;

extern string note                = "turn on Alert = true; turn off = false";
extern bool   alertsOn            = true;
extern bool   alertsOnZeroCross   = true;
extern bool   alertsOnSignalCross = true;
extern bool   alertsOnCurrent     = false;
extern bool   alertsMessage       = true;
extern bool   alertsSound         = true;
extern bool   alertsNotify        = false;
extern bool   alertsEmail         = false;

//-------------------------------
// Buffers
//-------------------------------

double FextMapBuffer1[];
double FextMapBuffer2[];
double FextMapBuffer3[];
double FextMapBuffer4[];
double FextMapBuffer5[];
double trend1[];
double trend2[];

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//|------------------------------------------------------------------|
int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,FextMapBuffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,FextMapBuffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,FextMapBuffer3);    
   SetIndexBuffer(3,FextMapBuffer4);  
   SetIndexBuffer(4,FextMapBuffer5);   
   SetIndexBuffer(5,trend1);   
   SetIndexBuffer(6,trend2);
   
      //
      //
      //
      //
      //

         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
      //
      //
      //
      //
      //  
   
   // My name
   IndicatorShortName(timeFrameToString(timeFrame)+  "  Heiken Ashi Oscillator");
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
  
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   int pos,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { FextMapBuffer1[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {     
     for(pos=limit; pos >= 0; pos--)
     {  
       double open  = iCustom(Symbol(), 0, "Heiken Ashi", 2, pos);
       double close = iCustom(Symbol(), 0, "Heiken Ashi", 3, pos);
      
       FextMapBuffer4[pos] = (open + close) / 2;
       FextMapBuffer5[pos] = FextMapBuffer4[pos] - FextMapBuffer4[pos+1];

       FextMapBuffer1[pos] = EMPTY_VALUE;
       FextMapBuffer2[pos] = EMPTY_VALUE;
       trend1[pos] = trend1[pos+1];
       if(FextMapBuffer5[pos]>0) trend1[pos] = 1;
       if(FextMapBuffer5[pos]<0) trend1[pos] =-1;
       if(trend1[pos] == 1) FextMapBuffer1[pos] = FextMapBuffer5[pos];
       if(trend1[pos] ==-1) FextMapBuffer2[pos] = FextMapBuffer5[pos];
   }
   
   // Apply average
   for(pos = limit; pos >=0; pos--) 
   {
      FextMapBuffer3[pos] = iMAOnArray(FextMapBuffer5,0,AvPeriod,0,MaMethod,pos);
      trend2[pos] = trend2[pos+1];
      if (FextMapBuffer5[pos]>FextMapBuffer3[pos]) trend2[pos] = 1;
      if (FextMapBuffer5[pos]<FextMapBuffer3[pos]) trend2[pos] =-1;
   }
   manageAlerts();
   return(0);
   }
   
   //
   //
   //
   //
   //
  
  limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
  for (pos=limit;pos>=0; pos--)
  {
     int y = iBarShift(NULL,timeFrame,Time[pos]);
        FextMapBuffer3[pos] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AvPeriod,MaMethod,2,y);
        FextMapBuffer4[pos] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AvPeriod,MaMethod,3,y);
        FextMapBuffer5[pos] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AvPeriod,MaMethod,4,y);
        trend1[pos]         = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AvPeriod,MaMethod,5,y);
        trend2[pos]         = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AvPeriod,MaMethod,6,y);
        FextMapBuffer1[pos] = EMPTY_VALUE;
        FextMapBuffer2[pos] = EMPTY_VALUE;
        if(trend1[pos] == 1) FextMapBuffer1[pos] = FextMapBuffer5[pos];
        if(trend1[pos] ==-1) FextMapBuffer2[pos] = FextMapBuffer5[pos];
  }
  manageAlerts();
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      
      //
      //
      //
      //
      //
      
      static datetime time1 = 0;
      static string   mess1 = "";
         if (alertsOnZeroCross && trend1[whichBar] != trend1[whichBar+1])
         {
            if (trend1[whichBar] ==  1) doAlert(time1,mess1,whichBar," Heiken Ashi Oscillator crossed zero up");
            if (trend1[whichBar] == -1) doAlert(time1,mess1,whichBar," Heiken Ashi Oscillator crossed zero down");
         }            
      static datetime time2 = 0;
      static string   mess2 = "";
         if (alertsOnSignalCross && trend2[whichBar] != trend2[whichBar+1])
         {
            if (trend2[whichBar] ==  1) doAlert(time2,mess2,whichBar," Heiken Ashi Oscillator crossed signal line up");
            if (trend2[whichBar] == -1) doAlert(time2,mess2,whichBar," Heiken Ashi Oscillator crossed signal line down");
         }            
    }
}   

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  Symbol()+" "+timeFrameToString(timeFrame)+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Heiken Ashi Oscillator "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}


   
   